#include "ImportedModel.h"

using namespace std;

ImportedModel::ImportedModel(char *sceneFile) {

	mFbxManager = 0;
	mFbxScene = 0;
	mBVHCount = 0;
	mGeometry.resize(0);

	vector<BoundingVolume *> primitives;
	primitives.resize(0); // just spheres
	char fbxFile[100];
	readSceneFle(sceneFile, primitives, fbxFile);
	
	if (strcmp(fbxFile, "") != 0) {
		InitializeSdkObjects(mFbxManager, mFbxScene);

		bool opened = LoadScene(mFbxManager, mFbxScene, fbxFile);

		if (opened) {
			printf("FBX successfully loaded\n");
		} else {
			printf("FBX failed to load\n");
		}

		// load models from fbx file
		loadGeometry();

		// done with fbx file
		DestroySdkObjects(mFbxManager);

		// get count
		for (unsigned int i = 0; i < mGeometry.size(); i++) {
			vector<Mesh *> meshes = mGeometry[i]->getMeshes();
			for (unsigned int j = 0; j < meshes.size(); j++) {
				++mBVHCount;
			}
		}
	}
	mVolumes = new BoundingVolume*[mBVHCount+primitives.size()];

	for (unsigned int i = 0; i < primitives.size(); i++) {
		mVolumes[i+mBVHCount] = primitives[i];
	}

	mBVHCount += (int)primitives.size();
	// construct bvh
	mLeafCount = 0;
	mPolyCount = 0;
	mHighestLeafCount = 0;
	constructHierarchy();
	calculateLeaves();

}

ImportedModel::~ImportedModel() {

	for (unsigned int i = 0; i < mGeometry.size(); i++) {
		delete mGeometry[i];
	}
	for (int i = 0; i < mBVHCount; i++) {
		deleteVolume(mVolumes[i]);
	}
	delete[] mVolumes;
	delete[] mLeafCount;

}

void ImportedModel::readSceneFle(char *sceneFile, vector<BoundingVolume *> &primitives, char *fbxFile) {
	ifstream infile(sceneFile);
	strcpy(fbxFile, "");
	if (infile) {
		string token;
		while (infile >> token) {
			// comments
			if (token.substr(0, 2).compare("//") == 0) {
				continue;
			}
			if (token.compare("FBX") == 0) {
				infile >> token;
				strcpy(fbxFile, token.c_str());
			} else if (token.compare("BEGIN_SPHERE") == 0) {
				float x = 1.0f, y = 0.0f, z = 0.0f;
				float r = 0.0f, g = 0.0f, b = 0.0f;
				float radius = 1.0f, reflection = 0.0f, refraction = 0.0f, index = 1.0f;
				while (token.compare("END_SPHERE")) {
					infile >> token;
					if (token.compare("CENTER") == 0) {
						infile >> x >> y >> z;
					} else if (token.compare("COLOR") == 0) {
						infile >> r >> g >> b;
					} else if (token.compare("RADIUS") == 0) {
						infile >> radius;
					} else if (token.compare("REFLECTION") == 0) {
						infile >> reflection;
					} else if (token.compare("REFRACTION") == 0) {
						infile >> refraction;
					} else if (token.compare("INDEX") == 0) {
						infile >> index;
					}
				}

				BoundingVolume *v = new BoundingVolume();
				v->mCenterX = x;
				v->mCenterY = y;
				v->mCenterZ = z;
				v->mRadius = radius;
				v->mR = r;
				v->mG = g;
				v->mB = b;
				v->mReflection = reflection;
				v->mRefraction = refraction;
				v->mIndexOfRefraction = index;
				v->mNumPolygons = 0;
				v->mNumVolumes = 0;
				v->mVolume1 = 0;
				v->mVolume2 = 0;
				v->mMatrix = 0;
				primitives.push_back(v);
			} else if (token.compare("BEGIN_POLY") == 0) {
				float x = 1.0f, y = 0.0f, z = 0.0f, nx = 0.0f, ny = 0.0f, nz = -1.0f;
				float r = 0.0f, g = 0.0f, b = 0.0f;
				float radius = 1.0f, reflection = 0.0f, refraction = 0.0f, index = 1.0f;
				vector<Vertex> verts;
				verts.resize(0);
				while (token.compare("END_POLY")) {
					infile >> token;
					if (token.compare("NORMAL") == 0) {
						infile >> nx >> ny >> nz;
					} else if (token.compare("VERTEX") == 0) {
						infile >> x >> y >> z;
						Vertex v;
						v.x = x;
						v.y = y;
						v.z = z;
						v.nx = nx;
						v.ny = ny;
						v.nz = nz;
						verts.push_back(v);
					} else if (token.compare("COLOR") == 0) {
						infile >> r >> g >> b;
					} else if (token.compare("REFLECTION") == 0) {
						infile >> reflection;
					} else if (token.compare("REFRACTION") == 0) {
						infile >> refraction;
					} else if (token.compare("INDEX") == 0) {
						infile >> index;
					}
				}

				BoundingVolume *v = new BoundingVolume();
				v->mR = r;
				v->mG = g;
				v->mB = b;
				v->mReflection = reflection;
				v->mRefraction = refraction;
				v->mIndexOfRefraction = index;
				v->mNumPolygons = 0;
				v->mNumVolumes = 0;
				v->mVolume1 = 0;
				v->mVolume2 = 0;
				v->mMatrix = 0;

				Polygon *p = new Polygon[1];
				Vertex *vertArray = new Vertex[verts.size()];
				for (unsigned int i = 0; i < verts.size(); i++) {
					vertexCopy(vertArray[i], verts[i]);
				}
				p->vertexCount = (int)verts.size();
				p->vertices = vertArray;
				p->r = r;
				p->g = g;
				p->b = b;
				p->reflection = reflection;
				p->refraction = refraction;
				p->indexOfRefraction = index;

				v->mNumPolygons = 1;
				fixNormals(p);
				v->mPolygons = p;

				vector<Polygon> polyVec;
				Polygon tempPoly;
				polygonCopy(tempPoly, *p);
				polyVec.push_back(tempPoly);
				createSphere(polyVec, x, y, z, radius);
				v->mCenterX = x;
				v->mCenterY = y;
				v->mCenterZ = z;
				v->mRadius = radius;

				primitives.push_back(v);
			 } else if (token.compare("LIGHT") == 0) {
				infile >> gLight.x >> gLight.y >> gLight.z;
			 }
		}



	}

	infile.close();
}

void ImportedModel::deleteVolume(BoundingVolume *v) {
	if (v->mNumVolumes != 0) {
		deleteVolume(v->mVolume1);
		deleteVolume(v->mVolume2);
	}
	delete v;
}
BoundingVolume **ImportedModel::getHierarchies() {
	return mVolumes;

}
void ImportedModel::loadGeometry() {

	mGeometry.resize(0);

	KFbxNode *root = mFbxScene->GetRootNode();
	if (mFbxScene) {
		if(root) {

			// get all children of the root
			for (int i = 0; i < root->GetChildCount(); i++) {
				KFbxNode *currNode = root->GetChild(i);

				// it's a mesh object
				if (currNode->GetNodeAttribute()->GetAttributeType() == KFbxNodeAttribute::eMESH) {
					Geometry *currGeometry = new Geometry(currNode);
					mGeometry.push_back(currGeometry);
				} else {  // see if it contains any children that are meshes
					searchTree(currNode);
				}
			}
		}

	}

}

void ImportedModel::searchTree(KFbxNode *node) {
	// get all children of the node that are meshes
	for (int i = 0; i < node->GetChildCount(); i++) {
		KFbxNode *currNode = node->GetChild(i);
		// it's a mesh object
		if (currNode->GetNodeAttribute()->GetAttributeType() == KFbxNodeAttribute::eMESH) {
			Geometry *currGeometry = new Geometry(currNode);
			mGeometry.push_back(currGeometry);
		} else {  // search its children
			for (int j = 0; j < currNode->GetChildCount(); j++) {
				searchTree(currNode->GetChild(j));
			}
		}
	}

}

void ImportedModel::calculateLeaves() {
	mLeafCount = new int[mBVHCount];
	mHighestLeafCount = 0;
	for (int i = 0; i < mBVHCount; i++) {
		mLeafCount[i] = numLeaves(mVolumes[i]);

		if (mLeafCount[i] > mHighestLeafCount) {
			mHighestLeafCount = mLeafCount[i];
		}
	}
}

int ImportedModel::numLeaves(BoundingVolume *v) {
	int total = 0;
	mPolyCount += v->mNumPolygons;
	if (v->mVolume1 != NULL && v->mVolume2 != NULL) {
		total += numLeaves(v->mVolume1) + numLeaves(v->mVolume2);
	} else { // this is a leaf
		total += 1;
	}

	return total;
}

void ImportedModel::constructHierarchy() {

	int index = 0;
	for (unsigned int i = 0; i < mGeometry.size(); i++) {
		vector<Mesh *> meshes = mGeometry[i]->getMeshes();
		for (unsigned int j = 0; j < meshes.size(); j++) {
			//printf("%s\n", meshes[j]->getName());
			vector<Polygon> polys = meshes[j]->getPolygons();
			mVolumes[index] = constructVolume(polys);
			mVolumes[index]->mMatrix = meshes[j]->getMatrix();
			index++;
		}
	}
}
int ImportedModel::getBVHCount() {
	return mBVHCount;
}

int ImportedModel::getNumberOfLeaves() {
	return mHighestLeafCount;
}
int ImportedModel::getNumberOfPolygons() {
	return mPolyCount;
}
BoundingVolume *ImportedModel::constructVolume(vector<Polygon> polygons) {
	BoundingVolume *parent = new BoundingVolume();

	if (polygons.size() <= MAX_POLYGONS) { // we've hit the threshold
		float x, y, z, r;
		createSphere(polygons, x, y, z, r);
		parent->mCenterX = x;
		parent->mCenterY = y;
		parent->mCenterZ = z;
		parent->mRadius = r;
		parent->mNumVolumes = 0;
		parent->mVolume1 = NULL;
		parent->mVolume2 = NULL;
		parent->mMatrix = NULL;
		parent->mNumPolygons = (int)polygons.size();

		// add polygons
		Polygon *polys = new Polygon[polygons.size()];
		for (unsigned int i = 0; i < polygons.size(); i++) {
			polygonCopy(polys[i], polygons[i]);;
		}

		parent->mPolygons = polys;

	} else { // split up the polygons into multiple volumes 
		float x, y, z, r;
		// get average point
		findAvg(polygons, x, y, z);
		// see which axis splits the polygons (not vertices) the most evenly
		int xAxis = 0, yAxis = 0, zAxis = 0;
		int xSpan = 0, ySpan = 0, zSpan = 0;

		for (unsigned int i = 0; i < polygons.size(); i++) {
			for (int j = 0; j < polygons[i].vertexCount; j++) {
				if (polygons[i].vertices[j].x > x) xAxis++;
				else xAxis--;
				if (polygons[i].vertices[j].y > y) yAxis++;
				else yAxis--;
				if (polygons[i].vertices[j].z > z) zAxis++;
				else zAxis--;
			}
			// if polygon spans both sides, that axis is not a good choice
			if (abs(xAxis) != polygons[i].vertexCount) xSpan++;
			if (abs(yAxis) != polygons[i].vertexCount) ySpan++;
			if (abs(zAxis) != polygons[i].vertexCount) zSpan++;
		}
		int spanWeight = 0;
		xAxis = abs(xAxis)+xSpan*spanWeight;
		yAxis = abs(yAxis)+ySpan*spanWeight;
		zAxis = abs(zAxis)+zSpan*spanWeight;

		// use the most "evenly dividing" axis to split them
		vector<Polygon> side1, side2;
		side1.resize(0);
		side2.resize(0);
		int axis = min(xAxis, min(yAxis, zAxis));

		for (unsigned int i = 0; i < polygons.size(); i++) {
			int s1 = 0, s2 = 0;
			for (int j = 0; j < polygons[i].vertexCount; j++) {
				if (axis == xAxis) {
					if (polygons[i].vertices[j].x < x) s1++;
					else s2++;
				} else if (axis == yAxis) {
					if (polygons[i].vertices[j].y < y) s1++;
					else s2++;
				} else if (axis == zAxis) {
					if (polygons[i].vertices[j].z < z) s1++;
					else s2++;
				}
			}
			// polygon might be entirely on one side, or might span both
			if (s1 != 0) {  // part of polygon is on side1
				side1.push_back(polygons[i]);
			}
			if (s2 != 0) {  // part of polygon is on side2
				side2.push_back(polygons[i]);
			}
		}
		// make sure both volumes are smaller than this one, 
		// or else the hierarchy will be infinitely deep
		if (side1.size() == polygons.size() || side2.size() == polygons.size()) {
			float x, y, z, r;
			createSphere(polygons, x, y, z, r);
			parent->mCenterX = x;
			parent->mCenterY = y;
			parent->mCenterZ = z;
			parent->mRadius = r;
			parent->mNumVolumes = 0;
			parent->mVolume1 = NULL;
			parent->mVolume2 = NULL;
			parent->mMatrix = NULL;
			parent->mNumPolygons = (int)polygons.size();

			// add polygons
			Polygon *polys = new Polygon[polygons.size()];
			for (unsigned int i = 0; i < polygons.size(); i++) {
				polygonCopy(polys[i], polygons[i]);;
			}

			parent->mPolygons = polys;

		} else {
			// get two volumes for these two groups of polygons
			BoundingVolume *v1 = constructVolume(side1);
			BoundingVolume *v2 = constructVolume(side2);

			// add them as children
			parent->mVolume1 = v1;
			parent->mVolume2 = v2;
			parent->mNumVolumes = 2;

			// get bounding sphere
			createSphere(polygons, x, y, z, r);

			parent->mCenterX = x;
			parent->mCenterY = y;
			parent->mCenterZ = z;
			parent->mRadius = r;
			parent->mNumPolygons = 0;

			parent->mPolygons = NULL;
		}
	}

	return parent;
}

void createSphere(vector<Polygon> polys, float &cx, float &cy, float &cz, float &r) {
	// find centroid by fitting a box 
	// around the object, then find center 
	// of the box
	float maxX = polys[0].vertices[0].x, maxY = polys[0].vertices[0].y, maxZ = polys[0].vertices[0].z;
	float minX = polys[0].vertices[0].x, minY = polys[0].vertices[0].y, minZ = polys[0].vertices[0].z;

	for (unsigned int i = 0; i < polys.size(); i++) {
		for (int j = 0; j < polys[i].vertexCount; j++) {
			if (maxX < polys[i].vertices[j].x) maxX = polys[i].vertices[j].x;
			if (maxY < polys[i].vertices[j].y) maxY = polys[i].vertices[j].y;
			if (maxZ < polys[i].vertices[j].z) maxZ = polys[i].vertices[j].z;

			if (minX > polys[i].vertices[j].x) minX = polys[i].vertices[j].x;
			if (minY > polys[i].vertices[j].y) minY = polys[i].vertices[j].y;
			if (minZ > polys[i].vertices[j].z) minZ = polys[i].vertices[j].z;
		}
	}

	// find center
	cx = (maxX + minX) / 2.0f;
	cy = (maxY + minY) / 2.0f;
	cz = (maxZ + minZ) / 2.0f;

	r = sqrt((maxX-minX)*(maxX-minX) + (maxY-minY)*(maxY-minY) + (maxZ-minZ)*(maxZ-minZ)) /2;

}

void findCenter(vector<Polygon> polys, float &cx, float &cy, float &cz) {
	// find centroid by fitting a box 
	// around the object, then find center 
	// of the box
	float maxX = polys[0].vertices[0].x, maxY = polys[0].vertices[0].y, maxZ = polys[0].vertices[0].z;
	float minX = polys[0].vertices[0].x, minY = polys[0].vertices[0].y, minZ = polys[0].vertices[0].z;

	for (unsigned int i = 0; i < polys.size(); i++) {
		for (int j = 0; j < polys[i].vertexCount; j++) {
			if (maxX < polys[i].vertices[j].x) maxX = polys[i].vertices[j].x;
			if (maxY < polys[i].vertices[j].y) maxY = polys[i].vertices[j].y;
			if (maxZ < polys[i].vertices[j].z) maxZ = polys[i].vertices[j].z;

			if (minX > polys[i].vertices[j].x) minX = polys[i].vertices[j].x;
			if (minY > polys[i].vertices[j].y) minY = polys[i].vertices[j].y;
			if (minZ > polys[i].vertices[j].z) minZ = polys[i].vertices[j].z;
		}
	}

	// find center
	cx = (maxX + minX) / 2.0f;
	cy = (maxY + minY) / 2.0f;
	cz = (maxZ + minZ) / 2.0f;
}

void findAvg(vector<Polygon> polys, float &cx, float &cy, float &cz) {
	float totalX = 0;
	float totalY = 0;
	float totalZ = 0;
	for (unsigned int i = 0; i < polys.size(); i++) {
		for (int j = 0; j < polys[i].vertexCount; j++) {
			totalX += polys[i].vertices[j].x;
			totalY += polys[i].vertices[j].y;
			totalZ += polys[i].vertices[j].z;
		}
	}

	cx = totalX / (float)polys.size();
	cy = totalY / (float)polys.size();
	cz = totalZ / (float)polys.size();
}

void ImportedModel::printBVH() {
	for (int i = 0; i < mBVHCount; i++) {
		printBVH(mVolumes[i], 0);
	}
	printf("Number of polygons: %d\n", mPolyCount);

}
void ImportedModel::printBVH(BoundingVolume *v, int level) {
	for (int i = 0; i < level; i++) {
		printf("\t");
	}
	// print this volume infomaation
	printf("Center: (%f,%f,%f)\tRadius: %f\n", v->mCenterX, v->mCenterY, v->mCenterZ, v->mRadius); 
	// print child information
	if (v->mNumVolumes != 0) {
		printBVH(v->mVolume1, level+1);
		printBVH(v->mVolume2, level+1);
	}
	// print polygon information
	if (v->mNumPolygons > 0) {
		for (int i = 0; i < (level+1); i++) {
			printf("\t");
		}
		printf("Number of Polygons: %d\n", v->mNumPolygons);
	}

}

unsigned int ImportedModel::getNumGeometry() {
	return (unsigned int)mGeometry.size();
}

Geometry *ImportedModel::getGeometry(unsigned int i) {
	if (i < 0 || i >= mGeometry.size()) {
		return NULL;
	}
	return mGeometry[i];
}

void ImportedModel::recreateHierarchy() {
	constructHierarchy();
}