#ifndef __ImportedModel__
#define __ImportedModel__

// contains ImportedModel data imported from fbx file
#include <fbxsdk.h>
#include <vector>
#include <math.h>
#include <string>
#include <fstream>

#include "fbx/Common.h"
#include "Mesh.h"
#include "Geometry.h"
#include "BoundingVolume.h"
#include "Vertex.h"

using namespace std;

// max number of polygons in a bounding volume
// if a mesh has more polys it will be broken
// into more than one volume
#define MAX_POLYGONS 50

// helper function that finds creates a sphere 
// that surrounds a number of polygons
void createSphere(vector<Polygon> polys, float &cx, float &cy, float &cz, float &r);
// find center, or centroid, of a number of polygons
void findCenter(vector<Polygon> polys, float &cx, float &cy, float &cz);
// find the average point
void findAvg(vector<Polygon> polys, float &cx, float &cy, float &cz);

class ImportedModel {

public:
	ImportedModel(char *sceneFile);
	~ImportedModel();

	BoundingVolume **getHierarchies();
	int getBVHCount();
	int getNumberOfLeaves();  // returns the highest out of all of them
	int getNumberOfPolygons();
	void printBVH();
	unsigned int getNumGeometry();
	Geometry *getGeometry(unsigned int i);
	void recreateHierarchy();

private:
	void readSceneFle(char *sceneFile, vector<BoundingVolume *> &primitives, char *fbxFile);
	void deleteVolume(BoundingVolume *v); // recursively delete volumes
	void loadGeometry();
	void searchTree(KFbxNode *node);  // recursively searches down from a node
									  // when a mesh is found it hands it off to Geometry
	// calculates number of polygns as well
	void calculateLeaves();
	int numLeaves(BoundingVolume *v);
	void constructHierarchy();
	BoundingVolume *constructVolume(vector<Polygon> polygons);  // create a "sub" hierarchy from a specific set of polygons
	void printBVH(BoundingVolume *v, int level);

	KFbxSdkManager *mFbxManager;
	KFbxScene *mFbxScene;
	vector<Geometry *> mGeometry;

	BoundingVolume **mVolumes;  // the actual bounding volume hierarchy
	int mBVHCount;
	int *mLeafCount;
	int mHighestLeafCount;
	int mPolyCount;

};

#endif
