#include "Util.h"
#include <stdio.h>
#include <math.h>

bool ENABLE_REFLECTION = true;		// enable reflection
bool ENABLE_REFRACTION = false;		// enable refraction / transmission


void printMatrix(Matrix &m) {
	printf("%f %f %f %f\n%f %f %f %f\n%f %f %f %f\n%f %f %f %f\n", m.c11, m.c12, m.c13, m.c14, 
		m.c21, m.c22, m.c23, m.c24, 
		m.c31, m.c32, m.c33, m.c34, 
		m.c41, m.c42, m.c43, m.c44);
}

float toRadians(float deg) {
	float rad = deg / 180.0f;
	rad *= 3.14159f;
	return rad;
}

// vector operations
// normalize
void normalize(float &xn, float &yn, float &zn) {
	float mag = sqrt(xn*xn + yn*yn + zn*zn);
	xn /= mag;
	yn /= mag;
	zn /= mag;
}
float3 crossProduct(float3 r1, float3 r2) {
	float3 c;
	c.x = r1.z*r2.y - r1.y*r2.z;
	c.y = r1.x*r2.z - r1.z*r2.x;
	c.z = r1.y*r2.x - r1.x*r2.y;

	return c;
}
// get an angle between two vectors
float getAngleBetween(float3 a, float3 b){
	float angle = a.x * b.x + a.y * b.y + a.z * b.z;
	return acos(angle);
	
}
// dot product
float dotProduct(float3 *r1, float3 *r2) {
	float dot = (r1->x*r2->x) +(r1->y*r2->y) +(r1->z*r2->z);
	return dot;
}

// matrix operations
void identityMatrix(Matrix &m) {
	m.c11 = 1.0f;
	m.c12 = 0.0f;
	m.c13 = 0.0f;
	m.c14 = 0.0f;

	m.c21 = 0.0f;
	m.c22 = 1.0f;
	m.c23 = 0.0f;
	m.c24 = 0.0f;

	m.c31 = 0.0f;
	m.c32 = 0.0f;
	m.c33 = 1.0f;
	m.c34 = 0.0f;

	m.c41 = 0.0f;
	m.c42 = 0.0f;
	m.c43 = 0.0f;
	m.c44 = 1.0f;
}

void translateMatrix(Matrix &m, float x, float y, float z) {
	m.c41 = x;
	m.c42 = y;
	m.c43 = z;

}

void rotateMatrix(Matrix &m, float x, float y, float z) {

	Matrix mx, my, mz;
	identityMatrix(mx);
	identityMatrix(my);
	identityMatrix(mz);

	rotateXMatrix(mx, x);
	rotateYMatrix(my, y);
	rotateZMatrix(mz, z);

	Matrix temp;
	multiplyMatrix(temp, mx, my);
	multiplyMatrix(m, temp, mz);

}

void rotateMatrix(Matrix &m, float x, float y, float z, float angle) {
	float s = sin(angle);
	float c = cos(angle);

	m.c11 = c + x*x*(1-c);
	m.c12 = x*y*(1-c) + z*s;
	m.c13 = x*z*(1-c) - y*s;
	m.c14 = 0.0f;

	m.c21 = x*y*(1-c) - z*s;
	m.c22 = c + y*y*(1-c);
	m.c23 = y*z*(1-c)+x*s;
	m.c14 = 0.0f;

	m.c31 = x*z*(1-c) + y*s;
	m.c32 = y*z*(1-c) - x*s;
	m.c33 = c + z*z*(1-c);
	m.c34 = 0.0f;

	m.c41 = 0.0f;
	m.c42 = 0.0f;
	m.c43 = 0.0f;
	m.c44 = 1.0f;
}
void scaleMatrix(Matrix &m, float x, float y, float z) {
	m.c11 = x;
	m.c22 = y;
	m.c33 = z;
}
void rotateXMatrix(Matrix &m, float a) {
	float s = sin(a);
	float c = cos(a);

	m.c22 = c;
	m.c32 = -s;
	m.c23 = s;
	m.c33 = c;

}
void rotateYMatrix(Matrix &m, float a) {
	float s = sin(a);
	float c = cos(a);
	m.c11 = c;
	m.c31 = s;
	m.c13 = -s;
	m.c33 = c;
}

void rotateZMatrix(Matrix &m, float a) {
	float s = sin(a);
	float c = cos(a);
	m.c11 = c;
	m.c21 = -s;
	m.c12 = s;
	m.c22 = c;
}
void multiplyMatrix(Matrix &out, const Matrix &m1, const Matrix &m2) {
	out.c11 = m1.c11*m2.c11 + m1.c21*m2.c12 + m1.c31*m2.c13+m1.c41*m2.c14;
	out.c21 = m1.c11*m2.c21 + m1.c21*m2.c22 + m1.c31*m2.c23+m1.c41*m2.c24;
	out.c31 = m1.c11*m2.c31 + m1.c21*m2.c32 + m1.c31*m2.c33+m1.c41*m2.c34;
	out.c41 = m1.c11*m2.c41 + m1.c21*m2.c42 + m1.c31*m2.c43+m1.c41*m2.c44;

	out.c12 = m1.c12*m2.c11 + m1.c22*m2.c12 + m1.c32*m2.c13+m1.c42*m2.c14;
	out.c22 = m1.c12*m2.c21 + m1.c22*m2.c22 + m1.c32*m2.c23+m1.c42*m2.c24;
	out.c32 = m1.c12*m2.c31 + m1.c22*m2.c32 + m1.c32*m2.c33+m1.c42*m2.c34;
	out.c42 = m1.c12*m2.c41 + m1.c22*m2.c42 + m1.c32*m2.c43+m1.c42*m2.c44;

	out.c13 = m1.c13*m2.c11 + m1.c23*m2.c12 + m1.c33*m2.c13+m1.c43*m2.c14;
	out.c23 = m1.c13*m2.c21 + m1.c23*m2.c22 + m1.c33*m2.c23+m1.c43*m2.c24;
	out.c33 = m1.c13*m2.c31 + m1.c23*m2.c32 + m1.c33*m2.c33+m1.c43*m2.c34;
	out.c43 = m1.c13*m2.c41 + m1.c23*m2.c42 + m1.c33*m2.c43+m1.c43*m2.c44;

	out.c14 = m1.c14*m2.c11 + m1.c24*m2.c12 + m1.c34*m2.c13+m1.c44*m2.c14;
	out.c24 = m1.c14*m2.c21 + m1.c24*m2.c22 + m1.c34*m2.c23+m1.c44*m2.c24;
	out.c34 = m1.c14*m2.c31 + m1.c24*m2.c32 + m1.c34*m2.c33+m1.c44*m2.c34;
	out.c44 = m1.c14*m2.c41 + m1.c24*m2.c42 + m1.c34*m2.c43+m1.c44*m2.c44;
}

void multiplyMatrixVector(Matrix &m, float &v1, float &v2, float &v3, float &v4) {
	float temp1 = v1*m.c11;
	temp1+= v2*m.c21;
	temp1+= v3*m.c31;
	temp1+= v4*m.c41;
	float temp2 = v1*m.c12 + v2*m.c22 + v3*m.c32 + v4*m.c42;
	float temp3 = v1*m.c13 + v2*m.c23 + v3*m.c33 + v4*m.c43;
	float temp4 = v1*m.c14 + v2*m.c24 + v3*m.c34 + v4*m.c44;

	v1 = temp1;
	v2 = temp2;
	v3 = temp3;
	v4 = temp4;
}
	
void getPos(Matrix &m, float &x, float &y, float &z) {
	x = m.c41;
	y = m.c42;
	z = m.c43;
}
float getScale(Matrix &m) {
	return m.c11;
}
