#ifndef __UTIL__
#define __UTIL__


#include <cuda_runtime_api.h>

// gloabl constants
#define MAX_DEPTH 1000 // bvh should not be deeper than this, though it's not enforced
extern bool ENABLE_REFLECTION;		// enable reflection
extern bool ENABLE_REFRACTION;		// enable refraction / transmission

extern float3 gLight;

//4x4 matrix
struct Matrix {
	float c11, c12, c13, c14;
	float c21, c22, c23, c24;
	float c31, c32, c33, c34;
	float c41, c42, c43, c44;
};


void printMatrix(Matrix &m);
float toRadians(float deg);

// vector operations
// normalize
void normalize(float &xn, float &yn, float &zn);
float3 crossProduct(float3 r1, float3 r2);
float getAngleBetween(float3 a, float3 b);
// dot product
float dotProduct(float3 *r1, float3 *r2);

// matrix operations
void identityMatrix(Matrix &m);
void translateMatrix(Matrix &m, float x, float y, float z);
void rotateMatrix(Matrix &m, float x, float y, float z); 
void rotateMatrix(Matrix &m, float x, float y, float z, float angle);  // axis-angle format
void scaleMatrix(Matrix &m, float x, float y, float z);
void rotateXMatrix(Matrix &m, float a);
void rotateYMatrix(Matrix &m, float a);
void rotateZMatrix(Matrix &m, float a);
void multiplyMatrix(Matrix &out, const Matrix &m1, const Matrix &m2);
void multiplyMatrixVector(Matrix &m, float &v1, float &v2, float &v3, float &v4);
// accessors
void getPos(Matrix &m, float &x, float &y, float &z);
float getScale(Matrix &m);

#endif